{ version 0.1						      }
{                                                             }
{ The color event was written and copyrighted by Keith Hawes. }
{ He can be contacted at khawes@ccmail.com.                   }
{                                                             }

unit Hgrid;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Grids;

type
  TGridColorEvent = procedure (Sender: TObject; ARow, ACol: Longint;
                               AState: TGridDrawState;
                               ABrush: TBrush; AFont: TFont ) of object;

  THGrid = class(TStringGrid)
  private
    { Private declarations }
    FHCol: TStrings;
    FHRow: TStrings;
    FOnGetCellColor: TGridColorEvent;
    procedure InitHCol;
    procedure InitHRow;
    procedure SetHCol(Value: TStrings);
    procedure SetHRow(Value: TStrings);
    procedure DrawCell(ACol, ARow : Longint; ARect : TRect;
      AState : TGridDrawState); override;
  protected
    { Protected declarations }
    procedure Loaded; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure MoveTo(ACol, ARow: longint);
  published
    { Published declarations }
    property HCol: TStrings read FHCol write SetHCol;
    property HRow: TStrings read FHRow write SetHRow;
    property OnGetCellColor: TGridColorEvent read FOnGetCellColor write FOnGetCellColor;
  end;

procedure Register;

implementation

uses NString;

procedure Register;
begin
  RegisterComponents('Samples', [THGrid]);
end;

constructor THGrid.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     FHCol:=TStringList.Create;
     FHRow:=TStringList.Create
end;

procedure THGrid.MoveTo(ACol, ARow: longint);
begin
     MoveColRow(ACol, ARow, True, True)
end;

procedure THGrid.InitHCol;
var
   I: Integer;
begin
     if (FHCol <> nil) then
     begin
          I:=0;
          while (I < FHCol.Count) and (I < ColCount) do
          begin
               Cells[I, 0]:=FHCol[I];
               inc(I)
          end
     end
end;

procedure THGrid.InitHRow;
var
   I: Integer;
begin
     if (FHRow <> nil) then
     begin
          I:=0;
          while (I < FHRow.Count) and (I < (ColCount - 1)) do
          begin
               Cells[0, I + 1]:=FHRow[I];
               inc(I)
          end
     end
end;

procedure THGrid.Loaded;
begin
     inherited Loaded;
     initHCol;
     initHRow
end;

procedure THGrid.SetHCol(Value: TStrings);
begin
     FHCol.Assign(Value);
     InitHCol;
     Refresh
end;

procedure THGrid.SetHRow(Value: TStrings);
begin
     FHRow.Assign(Value);
     InitHRow;
     Refresh
end;

procedure THGrid.DrawCell(ACol, ARow : Longint; ARect : TRect;
   AState : TGridDrawState);
var
   CellValue: String;
   Words: Array[1..5] of String;
   N, I, X, Y: Integer;
begin
     if Assigned(FOnGetCellColor) then
        FOnGetCellColor(Self, ARow, ACol, AState, Canvas.Brush, Canvas.Font);
     if ((ARow = 0) or (ACol = 0)) and ((HCol.Count > 0) or (HRow.Count > 0)) then
     begin
          CellValue:=Cells[ACol, ARow];
          N:=WordCount(CellValue, [';']);
          if N > 5 then N:=5;

          for I:=1 to N do
               Words[I]:=ExtractWord(I, CellValue, [';']);

          X:=ARect.Left + 2;
          Y:=ARect.Top;
          for I:=1 to N do
          begin
               Canvas.TextOut(X, Y, Words[I]);
               Y:=Y + Canvas.TextHeight(Words[I])
          end
     end
     else
         inherited DrawCell(ACol, ARow, ARect, AState)
end;

end.
